# /*=========================================================================
#    Program: ODB2VTK
#    Module:  multiprocess.py
#    Copyright (c) Arris Composites Inc.
#    All rights reserved.
#  
#    Arris Composites Inc.
#    745 Heinz Ave
#    Berkeley, CA 94710
#    USA
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ========================================================================*/

# Command line tool to spawn 'abaqus python' call in parallel

import multiprocessing
import subprocess
import argparse
import sys
import os
import odb2vtk

def spawn(cmd):
    return subprocess.call(cmd, shell=True)

# use multiprocessing to spawn abaqus python call in parallel
if __name__ == "__main__":
    script_dir = os.path.abspath( os.path.dirname( __file__ ) )

    parser = argparse.ArgumentParser()
    parser.add_argument("--writeHistory", default=0, type=int, help="if 1, write history output.")
    parser.add_argument("--odbFile", required=True, help="selected odb file (full path name)")
    args = parser.parse_args()

    # check odbfile
    if not args.odbFile:
        sys.exit("Need an .odf file as input")
    if not os.path.exists(args.odbFile):
        sys.exit("{0} doesn't exist".format(args.odbFile))
	
    odb2vtkObj = odb2vtk.ODB2VTK(args.odbFile, "")

    instances = odb2vtkObj.getInstances()
    #{'stepname': [0, 1, 2, 3], 'stepname': [0,1,2]}
    steps = odb2vtkObj.getSteps()
    odb2vtkObj.WritePVDFileAll()
    
    cmd = []
    for instance in instances:
            print("write instance: "+instance)
            for stepName in steps:
                for frame in steps[stepName]: 
                    step_frame_dict = "{0}:{1}".format(stepName,frame)
                    cmd.append('abaqus python {0}/odb2vtk.py --odbFile {1} --instance {2} --step "{3}" --postfix {4} --writeHistory {5}'
                        .format(script_dir, args.odbFile, instance, step_frame_dict, instance, args.writeHistory))

    count = multiprocessing.cpu_count()
    pool = multiprocessing.Pool(processes=count)
    pool.map(spawn, cmd)
